@@ -1,16 +1,13 @@
-# $Id: RemoveVOBs.pm,v 1.6 2005/12/26 13:57:47 joern Exp $
+# $Id: RemoveVOBs.pm,v 1.1 2002/03/03 15:09:28 joern Exp $
 
 #-----------------------------------------------------------------------
-# Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
-# All Rights Reserved. See file COPYRIGHT for details.
+# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
 # 
 # This program is part of Video::DVDRip, which is free software; you can
 # redistribute it and/or modify it under the same terms as Perl itself.
 #-----------------------------------------------------------------------
 
 package Video::DVDRip::Cluster::Job::RemoveVOBs;
-use Locale::TextDomain qw (video.dvdrip);
-use Video::DVDRip::FixLocaleTextDomainUTF8;
 
 use base Video::DVDRip::Cluster::Job;
 
@@ -21,7 +18,7 @@ sub info {
 	return "remove vob files";
 }
 
-sub command {
+sub start {
 	my $self = shift;
 
 	my $project = $self->project;
@@ -32,17 +29,31 @@ sub command {
 	my $command = $title->get_remove_vobs_command;
 	$project->set_assigned_job ( undef );
 	
-	return $command;
-}
-
-sub parse_output {
-	my $self = shift;
-	my ($line) = @_;
-	
-	$self->set_operation_successful ( 1 )
-		if $line =~ /DVDRIP_SUCCESS/;
+	my $successful_finished = 0;
+	$self->popen (
+		command      => $command,
+		cb_line_read => sub {
+			my ($line) = @_;
+			if ( $line =~ /DVDRIP_SUCCESS/ ) {
+				$successful_finished = 1;
+			}
+		},
+		cb_finished  => sub {
+			if ( $successful_finished ) {
+				$self->commit_job;
+			} else {
+				$self->abort_job;
+			}
+		},
+	);
 
 	1;
 }
 
+sub calc_progress {
+	my $self = shift;
+
+	return "removing VOB files";
+}
+
 1;
