@@ -1,4 +1,4 @@
-# $Id: Config.pm,v 1.6 2002/01/03 17:40:00 joern Exp $
+# $Id: Config.pm,v 1.9 2002/03/02 16:20:04 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
@@ -28,50 +28,70 @@ sub last_saved_data		{ shift->{last_saved_data}		}
 sub set_filename		{ shift->{filename}		= $_[1] }
 sub set_last_saved_data		{ shift->{last_saved_data}	= $_[1] }
 sub set_presets			{ shift->{presets}		= $_[1] }
+sub set_order			{ shift->{order}		= $_[1] }
+
+my %CONFIG_PARAMETER = (
+	program_name => {
+		type  => 'string',
+		value => "dvd::rip",
+	},
+	main_window_width => {
+		label => "Startup Window Width",
+		type  => 'number',
+		value => 660,
+	},
+	main_window_height => {
+		label => "Startup Window Height",
+		type  => 'number',
+		value => 650,
+	},
+	thumbnail_factor => {
+		type  => 'number',
+		value => 5,
+	},
+	dvd_device => {
+		label => "DVD Device",
+		type => 'file',
+		value => "/dev/dvd",
+	},
+	base_project_dir => {
+		label => "Default Data Base Directory",
+		type => 'dir',
+		value => "/spare/dvdrip",
+	},
+	cluster_master_local => {
+		label => "Start Cluster Control Daemon locally",
+		type  => 'switch',
+		value => 1,
+	},
+	cluster_master_server => {
+		label => "Hostname of server with Cluster Control Daemon",
+		type  => 'string',
+		value => "",
+	},
+	cluster_master_port => {
+		label => "TCP Port Number of Cluster Control Daemon",
+		type  => 'number',
+		value => 28646,
+	},
+);
+
+my @CONFIG_ORDER = qw (
+	dvd_device base_project_dir
+	main_window_width main_window_height
+	cluster_master_local cluster_master_server
+	cluster_master_port
+);
 
 sub new {
 	my $type = shift;
 
+	my %config_parameter = %CONFIG_PARAMETER;
+	my @config_order     = @CONFIG_ORDER;
+
 	my $self = {
-		config => {
-			program_name => {
-				type  => 'string',
-				value => "dvd::rip",
-			},
-			main_window_width => {
-				label => "Startup Window Width",
-				type  => 'number',
-				value => 660,
-			},
-			main_window_height => {
-				label => "Startup Window Height",
-				type  => 'number',
-				value => 650,
-			},
-			thumbnail_factor => {
-				type  => 'number',
-				value => 5,
-			},
-			dvd_device => {
-				label => "DVD Device",
-				type => 'file',
-				value => "/dev/dvd",
-			},
-			dvd_mount_point => {
-				label => "DVD Mount Point Directory",
-				type => 'dir',
-				value => "/media/dvd",
-			},
-			base_project_dir => {
-				label => "Default Data Base Directory",
-				type => 'dir',
-				value => "/spare/dvdrip",
-			},
-		},
-		order => [qw(
-			dvd_device dvd_mount_point base_project_dir
-			main_window_width main_window_height
-		)],
+		config => \%config_parameter,
+		order  => \@config_order,
 		presets => [
 			Video::DVDRip::Preset->new (
 				name => "nopreset",
@@ -191,17 +211,16 @@ sub load {
 	confess "no filename set" if not $filename;
 	confess "can't read $filename" if not -r $filename;
 	
-	my $config;
-	$config = do $filename;
+	my $loaded;
+	$loaded = do $filename;
 	confess "can't load $filename. Perl error: $@" if $@;
 
-	my $presets = $self->presets;
-
-	%{$self} = %{$config};
-	
-	# actually we overide presets, since no user editable
-	# presets exist
-	$self->set_presets ($presets);
+	foreach my $par ( @{$self->order} ) {
+		if ( exists $loaded->config->{$par} ) {
+			$self->config->{$par}->{value} =
+				$loaded->config->{$par}->{value};
+		}
+	}
 	
 	1;
 }
@@ -232,7 +251,7 @@ sub save {
 	my $fh = FileHandle->new;
 
 	open ($fh, "> $filename") or confess "can't write $filename";
-	print $fh q{# $Id: Config.pm,v 1.6 2002/01/03 17:40:00 joern Exp $},"\n";
+	print $fh q{# $Id: Config.pm,v 1.9 2002/03/02 16:20:04 joern Exp $},"\n";
 	print $fh "# This file was generated by Video::DVDRip Version $Video::DVDRip::VERSION\n\n";
 
 	print $fh ${$data_sref};
