@@ -1,7 +1,6 @@
-# $Id: Window.pm,v 1.4 2004/04/11 23:36:20 joern Exp $
+# $Id: Window.pm,v 1.1 2001/11/25 10:47:12 joern Exp $
 
 package Video::DVDRip::GUI::Window;
-use Locale::TextDomain qw (video.dvdrip);
 
 @ISA = qw ( Video::DVDRip::GUI::Component );
 
@@ -51,14 +50,10 @@ sub instance_closed {
 sub open_window {
 	my $self = shift;
 
-	if ($self->single_instance_window and
-	    $self->gtk_window_widget ) {
-		$self->gtk_window_widget->hide();
-		$self->gtk_window_widget->show();
-		return;
-	}
-
-	$self->build(@_) or return 1;
+	return if $self->single_instance_window and
+	          $self->gtk_window_widget;
+	
+	$self->build(@_);
 
 	confess ("Window component didn't set gtk_window_widget")
 		if not $self->gtk_window_widget;
